/*************************************************************************/
/* libsurl - library for generating shortened URLs                       */
/* Copyright (C) 2009  Scott Lawrence <bytbox@gmail.com> et al.          */
/*                                                                       */
/* This program is free software: you can redistribute it and/or modify  */
/* it under the terms of the GNU General Public License as published by  */
/* the Free Software Foundation, either version 3 of the License, or (at */
/* your option) any later version.                                       */
/*                                                                       */
/* This program is distributed in the hope that it will be useful, but   */
/* WITHOUT ANY WARRANTY; without even the implied warranty of            */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     */
/* General Public License for more details.                              */
/*                                                                       */
/* You should have received a copy of the GNU General Public License     */
/* along with this program.  If not, see <http://www.gnu.org/licenses/>. */
/*************************************************************************/

/* surl.c - main source file for libsurl */

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <pthread.h>

#include "config.h"
#include "surl.h"
#include "engine.h"

SURL *surl_init() {
    //allocate
    SURL *surl=(SURL *)malloc(sizeof(surl_t));
    int i;

    //make all the options null
    for(i=0;i<SURLOPT_COUNT;i++)
        surl->options[i]=0;

    //init boolean and counting variables to 0
    surl->engine_cnt=0;
    surl->threading=0;
    surl->error=0;
    surl->debug=0;
    surl_register_builtin_engines(surl);
    return surl;
}

int surl_register_engine(SURL *surl,char *service,surl_engine_t *engine) {
    if(surl_get_engine(surl,service)) // we already have an engine registered
        return -1;
    if(surl->engine_cnt==SURL_MAX_SERVICES) // we have too many engines registered
        return -1;
    puts(service);
    char *stmp=(char *)malloc(strlen(service)+3);
    sprintf(stmp,"%s",service);
    surl->services[surl->engine_cnt]=stmp;
    surl->engines[surl->engine_cnt]=engine;
    surl->engine_cnt++;
    return 0;
}

surl_engine_t *surl_get_engine(SURL *surl,char *service) {
    int i=0;
    for(i=0;i<surl->engine_cnt;i++) {
        if(!strcmp(service,surl->services[i]))
            return surl->engines[i];
    }
    return 0;
}

int surl_get_error(SURL *surl) {
    return surl->error;
}

void surl_reset_error(SURL *surl) {
    surl->error=0;
}

void surl_debug_on(SURL *surl) {
    surl->debug=1;
}

void surl_debug_off(SURL *surl) {
    surl->debug=0;
}

int surl_debug(SURL *surl) {
    return surl->debug;
}

void surl_setopt(SURL *surl,int opt,char *val) {
    if(surl->options[opt])
        free(surl->options[opt]);
    if(val==0) val="";
    surl->options[opt]=(char *)malloc(strlen(val)+2);
    sprintf(surl->options[opt],"%s",val);
}

char *surl_getopt(SURL *surl,int opt) {
    if(surl->options[opt]) {
        char *tmp=(char *)malloc(strlen(surl->options[opt])+2);
        sprintf(tmp,"%s",surl->options[opt]);
        return tmp;
    }
    return "";
}

void surl_cleanup(SURL *surl) {
    int i;
    for(i=0;i<SURLOPT_COUNT;i++)
        if(surl->options[i])
            free(surl->options[i]);
    for(i=0;i<surl->engine_cnt;i++)
        if(surl->services[i])
            free(surl->services[i]);
    free(surl);
}


/************************/
/* Simple Use Functions */
/************************/

char *surl_simple_shorten(char *service,char *url) {
    return surl_do_shorten(service,url,0,0,0,0);
}

char *surl_do_shorten(char *service,char *url,char *title,char *username,char *password,char *apikey) {
    SURL *surl=surl_init();
    surl_setopt(surl,SURLOPT_SERVICE,service);
    surl_setopt(surl,SURLOPT_URL,url);
    surl_setopt(surl,SURLOPT_TITLE,title);
    surl_setopt(surl,SURLOPT_USERNAME,username);
    surl_setopt(surl,SURLOPT_PASSWORD,password);
    surl_setopt(surl,SURLOPT_API_KEY,apikey);
    surl_shorten(surl);
    surl_wait(surl);
    return surl_getopt(surl,SURLOPT_SHORT_URL);
}


void *surl_shorten0(void *s) {
    SURL *surl=(SURL *)s;
    char *service=surl_getopt(surl,SURLOPT_SERVICE);
    if(service==0) {
        surl->error=SURLERR_SERVICE;
        return 0;
    }

    surl_engine_t *engine=surl_get_engine(surl,service);
    if(engine==0) {
        surl_setopt(surl,SURLOPT_SHORT_URL,surl_getopt(surl,SURLOPT_URL));
        surl->error=SURLERR_SERVICE;
        return 0;
    }
    engine->shortener(surl,engine->data);

    surl->threading=0;
    return 0;
}

int surl_shorten(SURL *surl) {
    if(surl->threading) return -1;
    surl->threading=1;
    pthread_create(&surl->pthread,0,surl_shorten0,(void *)surl);
    return 0;
}

void surl_wait(SURL *surl) {
    if(surl->threading!=0)
        pthread_join(surl->pthread,0);
    surl->threading=0;
}

void surl_set_callback(SURL *surl,surl_callback_t callback) {
    surl->callback=callback;
}

surl_callback_t surl_get_callback(SURL *surl) {
    return surl->callback;
}

char *libsurl_version() {
    return PACKAGE_VERSION;
}

char *libsurl_bugreport() {
    return PACKAGE_BUGREPORT;
}

char *libsurl_config_date() {
    return CONFIG_DATE;
}

char *libsurl_build_date() {
    char *tmp=(char *)malloc(strlen(__TIME__)+strlen(__DATE__)+3);
    sprintf(tmp,"%s %s",__DATE__,__TIME__);
    return tmp;
}
